package com.almworks.jira.structure.customitemtype;

import com.almworks.jira.structure.api.attribute.AttributeValue;
import com.almworks.jira.structure.api.attribute.CoreAttributeSpecs;
import com.almworks.jira.structure.api.attribute.loader.ItemAttributeContext;
import com.almworks.jira.structure.api.attribute.loader.basic.ItemClassAttributeLoader;
import com.almworks.jira.structure.api.attribute.loader.basic.SimpleAttributeProvider;
import com.atlassian.jira.project.Project;
import com.opensymphony.util.TextUtils;
import org.jetbrains.annotations.NotNull;

/**
 * This class demonstrates how to provide a displayable name and an icon of a custom entity in the Structure's standard
 * {@link CoreAttributeSpecs#SUMMARY Summary} column.
 */
public class CustomProjectLoaderProvider extends SimpleAttributeProvider {
  public CustomProjectLoaderProvider() {
    // Show project name in the standard Summary column
    registerLoader(new ItemClassAttributeLoader<Project, String>(CoreAttributeSpecs.SUMMARY, Project.class, CustomProjectItemType.KEY) {
      @Override
      protected AttributeValue<String> getValue(@NotNull Project item, ItemAttributeContext context) {
        return AttributeValue.of(item.getName());
      }
    });
    // Show project icon (right before project name in the Summary column)
    registerLoader(new ItemClassAttributeLoader<Project, String>(CoreAttributeSpecs.ICON, Project.class, CustomProjectItemType.KEY) {
      @Override
      protected AttributeValue<String> getValue(@NotNull Project project, ItemAttributeContext context) {
        return AttributeValue.of(String.format(
          "<img src=\"%s/secure/projectavatar?pid=%d&size=small\" alt=\"%s\" title=\"%s\">",
          context.getBaseUrl(), project.getId(), TextUtils.htmlEncode(project.getKey()), TextUtils.htmlEncode(project.getName())));
      }
    });
  }
}
