package com.almworks.structure.foosync;

import com.almworks.integers.LongList;
import com.almworks.jira.structure.api.StructureComponents;
import com.almworks.jira.structure.api.error.StructureException;
import com.almworks.jira.structure.api.forest.ForestSource;
import com.almworks.jira.structure.api.forest.raw.Forest;
import com.almworks.jira.structure.api.sync.*;
import com.almworks.jira.structure.api.util.MapObject;
import com.almworks.jira.structure.api.util.StructureUtil;
import com.atlassian.jira.web.action.JiraWebActionSupport;
import com.google.common.collect.ImmutableMap;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * The main class of the synchronizer - it contains a lot of describing methods
 * and the main methods resync() and sync()
 */
public class FooSynchronizer extends AbstractSynchronizer {
  private static final Logger logger = LoggerFactory.getLogger(FooSynchronizer.class);

  public FooSynchronizer(StructureComponents structureComponents) {
    super(structureComponents);
  }

  public boolean isAvailable() {
    return true;
  }

  public boolean isAutosyncSupported() {
    return true;
  }

  public String getConfigDescription(@Nullable Object parameters) {
    Map<String, Object> params = castParameters(parameters);
    String fooId = params == null ? "" : new MapObject(params).getString("fooId");
    return "Foo: " + fooId;
  }

  public List<String> getConfigDescriptionDetails(@Nullable Object parameters) {
    return Collections.emptyList();
  }

  @Nullable
  @Override
  public List<String> getPossibleResyncEffects(@Nullable Object parameters) {
    // Write possible effects: e.g., "change structure", or "change Foo", or something more detailed
    return Collections.singletonList("Change structure");
  }

  public void addDefaultFormParameters(@NotNull Map<String, Object> params) {
    params.put("fooId", "default-foo-id");
  }

  public Object buildParametersFromForm(@NotNull Map<String, ?> formParameters, @NotNull JiraWebActionSupport action) {
    String fooId = StructureUtil.getSingleParameter(formParameters, "fooId");
    if (fooId == null || fooId.trim().length() == 0) {
      action.addError("fooId", "Please specify non-empty foo ID");
      return null;
    }
    return ImmutableMap.of("fooId", fooId);
  }

  @Override
  public void addFormParameters(@Nullable Object syncParams, @NotNull Map<String, Object> formParams) {
    String fooId = parametersAsNNMapObject(syncParams).getString("fooId");
    if (fooId != null) {
      formParams.put("fooId", fooId);
    }
  }

  @NotNull
  @Override
  protected SyncRunAuditEntry doSync(@NotNull SyncInstance instance,
    @NotNull IncrementalSyncData data, @NotNull ForestSource forestSource) throws StructureException
  {
    // todo -- implement actual incremental sync

    LongList issues = data.getStructureChangedIssuesSorted();
    // now resync only those issues
    logger.warn("INCREMENTAL SYNC from STRUCTURE to FOO: " + issues);

    // Return the list describing the applied actions
    return success(Collections.<MapObject>emptyList());
  }

  @NotNull
  @Override
  protected SyncRunAuditEntry doResync(@NotNull SyncInstance instance,
    @NotNull ForestSource forestSource) throws StructureException
  {
    MapObject parameters = parametersAsNNMapObject(instance.getParameters());
   // If the synchronizer always works in one direction:
    return resyncFromStructureToFoo(forestSource, parameters);

    // If it works in both directions, add a direction parameter and check it here. See also SourceOfTruth.
  }

  private SyncRunAuditEntry resyncFromStructureToFoo(ForestSource forestSource, MapObject parameters) {
    // todo -- implement actual resync
    Forest forest = forestSource.getLatest().getForest();
    for (int i = 0; i < forest.size(); i++) {
      long rowId = forest.getRow(i);
      int depth = forest.getDepth(i);
      logger.warn("RESYNC from STRUCTURE to FOO: " + rowId + ":" + depth);
    }

    // Return the list describing the applied actions
    return success(Collections.<MapObject>emptyList());
  }

}
