package com.almworks.jira.structure.apiclient;

import com.almworks.jira.structure.api.StructureComponents;
import com.almworks.jira.structure.api.query.StructureQuery;
import com.almworks.jira.structure.api.query.StructureQueryBuilder;
import com.atlassian.jira.jql.builder.JqlQueryBuilder;
import com.atlassian.jira.plugin.workflow.WorkflowPluginConditionFactory;
import com.atlassian.query.Query;
import com.google.common.collect.Maps;
import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.workflow.WorkflowException;
import com.opensymphony.workflow.loader.AbstractDescriptor;

import java.util.Collections;
import java.util.Map;

public class PrevSiblingsResolvedCondition extends AbstractStructureQueryCondition implements WorkflowPluginConditionFactory {
  private final StructureQuery myHasUnresolvedPrev;

  public PrevSiblingsResolvedCondition(StructureComponents structureComponents) {
    super(structureComponents.getStructureManager());
    
    Query unresolved = JqlQueryBuilder.newBuilder().where().unresolved().buildQuery();
    myHasUnresolvedPrev = StructureQueryBuilder.begin().prevSibling.in.jql(unresolved).end();
    // Alternative:
//    myHasUnresolvedPrev = structureComponents.getStructureQueryBuilderFactory().builder().prevSibling.in.jql(unresolved).end();
  }

  @Override
  public boolean passesCondition(Map transientVars, Map args, PropertySet ps) throws WorkflowException {
    return !matchesIssue(myHasUnresolvedPrev, transientVars, args);
  }

  @Override
  public Map<String, ?> getVelocityParams(String resourceName, AbstractDescriptor descriptor) {
    return Maps.newHashMap();
  }

  @Override
  public Map<String, ?> getDescriptorParams(Map<String, Object> formParams) {
    return Collections.EMPTY_MAP;
  }
}
